/*
 * morse.h
 *  Morse encode - decode functions
 *
 *  Created on: 24/11/2013
 *  Author:  Samuel Melero Cuesta
 *  Version: 1.0
 *
 */
#ifndef MORSE_H_
#define MORSE_H_

#define MORSE_DASH			( 0 )					/* - */
#define MORSE_DOT 			( MORSE_DASH + 1 )		/* . */
#define MORSE_DASH_CHAR		( MORSE_DASH + 2 )		/* dash of new char */
#define MORSE_DOT_CHAR		( MORSE_DASH + 3 )		/* dot of new char */
#define MORSE_DOT_WORD		( MORSE_DASH + 4 )		/* dot of new word */
#define MORSE_DASH_WORD		( MORSE_DASH + 5 )		/* dash of new word */
#define MORSE_NULL			( MORSE_DASH + 6 )		/* Not a Morse symbol */
#define MORSE_PAUSE_MAX		( 1000 )				/* One second max pause */
#define MORSE_DOT_TIME		( 100 )					/* MAX Dot time in msecs */
#define MORSE_DASH_TIME		( 3 * MORSE_DOT_TIME )	/* MAX Dash time in msecs */
#define MORSE_CHAR_PAUSE	( 120 )					/* MAX pause time between dots and dashes in msecs */

#define MORSE_WPM_CONST		( 1200 )				/* 1440 const standard */
#define MORSE_TIME_RANGE	( 0.15 )				/* 15% range admitted time */

#define MORSE_FREQ_DEF		( 900UL )				/* 900 Hz freq default value */
#define MORSE_WPM_DEF		( 14UL )				/* 14 wpm default value */
#define MORSE_VOLUME_DEF	( 50 )					/* 50% volume default value */


/*****************************************************************************
** Function name:		initMorseDetector
**
** Descriptions:		Initializes and configures Morse detector.
**
** parameters:			receiverQueue, the queue that holds detector symbols.
** Returned value:		None
**
*****************************************************************************/
void initMorseReceiver(xQueueHandle receiverQueue);

/*****************************************************************************
** Function name:		initMorseTransmitter
**
** Descriptions:		Initializes and configures Morse sound transmitter
** 						with default values.
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void initMorseTransmitter();

/*****************************************************************************
** Function name:		setMorseOptions
**
** Descriptions:		Set Morse receiver & transmitter options.
**
** parameters:			String containing Morse configuration
** Returned value:		None
**
*****************************************************************************/
void setMorseOptions(char* options);

/*****************************************************************************
** Function name:		getMorseOptions
**
** Descriptions:		Get Morse receiver & transmitter current options.
**
** parameters:			Morse sound frequency, words per minute speed, %volume
** Returned value:		None
**
*****************************************************************************/
void getMorseOptions(char* string);

/*****************************************************************************
** Function name:		calcWpmTimeBase
**
** Descriptions:		Calc of the Morse dot, dash and pause times from WPM parameter.
**
** parameters:			wpm (Words Per Minute).
** Returned value:		None
**
*****************************************************************************/
void calcWpmTimeBase(uint32_t wpm);

/*****************************************************************************
** Function name:		getMorseWordsPauseTime
**
** Descriptions:		Returns a litle large pause time between words.
** 						This function is used in main application to
** 						send last char in Morse buffer.
**
** parameters:			None
** Returned value:		mseg between words + 2*dash time
**
*****************************************************************************/
portTickType getMorseWordsPauseTime();

/*****************************************************************************
** Function name:		charPending
**
** Descriptions:		Check if a character is pending in morse buffer.
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
uint32_t charPending();

/*****************************************************************************
** Function name:		addMorseDot
**
** Descriptions:		Add new dot to morse binary code.
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void addMorseDot();

/*****************************************************************************
** Function name:		addMorseDash
**
** Descriptions:		Add new dash to morse binary code.
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void addMorseDash();

/*****************************************************************************
** Function name:		getMorseSymbol
**
** Descriptions:		Returns dot or dash of new char or current char.
**
** parameters:			pauseTime, signalTime
** Returned value:		MORSE_DASH, MORSE_DOT, MORSE_DASH_CHAR, MORSE_DOT_CHAR
**
*****************************************************************************/
uint8_t getMorseSymbol(uint32_t pauseTime, uint32_t signalTime);

/*****************************************************************************
** Function name:		decodeChar
**
** Descriptions:		Decode and retrieve character pending from
** 						binary Morse code buffer.
**
** parameters:			None
** Returned value:		Decoded character
**
*****************************************************************************/
char decodeChar();

/*****************************************************************************
** Function name:		encodeChar
**
** Descriptions:		Encode char to Morse and retrieve code symbols.
**
** parameters:			charToEncode, string to store Morse code symbols
** Returned value:		Num of Morse code symbols stored in string
**
*****************************************************************************/
uint32_t encodeChar(char charToEncode, char* string);

/*****************************************************************************
** Function name:		doMorseSound
**
** Descriptions:		Encode character and throw pwm Morse signal on PIN 46.
**
** parameters:			charToEncode, char to encode to Morse and make sound
** Returned value:		None
**
*****************************************************************************/
void doMorseSound(char charToEncode);

#endif /* MORSE_H_ */
