/*
 * wifly.h
 *  Wifly driver
 *
 *  Created on: 24/10/2013
 *  Author:  Samuel Melero Cuesta
 *  Version: 2.0
 *
 */

#ifndef WIFLY_H_
#define WIFLY_H_

/* Wifly hardware configuration defines */
#define RST_WIFLY_PIN				( 22UL )
#define RST_WIFLY_BIT				( 12UL )

#define WLAN_WIFLY_BIT				( 22UL )
#define WLAN_WIFLY_PIN				( 27UL )
#define TCP_WIFLY_BIT				( 24UL )
#define TCP_WIFLY_PIN				( 28UL )

/* Wifly lib defines */
#define WF_MAX_CMD_SIZE				( 512UL )
#define WF_STDCOMM_TIME				( 250UL )
#define WF_JOIN_TIME				( 10000UL )
#define WF_RESET_TIME				( 4000UL )
#define WF_OPENTCP_TIME				( 3000UL )
#define WF_SET_PARMS				( "%s\r" )
#define WF_CMD_ENTER				( "$$$" )
#define WF_CMD_EXIT					( "exit" )
#define WF_IP_INI					("IP=")
#define WF_IP_END					(" :")
#define WF_CMD_ENTER_RESP			("CMD")
#define WF_CMD_EXIT_RESP			("EXIT")
#define WF_CMD_JOIN					("join ")
#define WF_SET_WEP_KEY				("set wlan num")
#define WF_SET_WPA_PASS				("set wlan phrase")
#define WF_GET_IP					("get ip")
#define WF_WEP_MODE					( 0UL )
#define WF_WPA_MODE					( 1UL )
#define WF_NO_PROTECTED				( 2UL )


/*****************************************************************************
** Function name:		wiflyGetBuffer
**
** Descriptions:		Retrieves wifly received data
**
** parameters:			String to receive data, time to listen for data
** Returned value:		Number of chars received
**
*****************************************************************************/
uint32_t wiflyGetBuffer(char* string, uint32_t timeout);

/*****************************************************************************
** Function name:		wiflyGetTCPStatus
**
** Descriptions:		Retrieve the TCP GPIO PIN state from wifly
**
** parameters:			None
** Returned value:		TRUE if TCP connection is up, FALSE otherwise
**
*****************************************************************************/
uint32_t wiflyGetTCPStatus();

/*****************************************************************************
** Function name:		wiflyGetWLANStatus
**
** Descriptions:		Retrieve the WLAN GPIO PIN state from wifly
**
** parameters:			None
** Returned value:		TRUE if wifly is joined to SSID, FALSE otherwise
**
*****************************************************************************/
uint32_t wiflyGetWLANStatus();

/*****************************************************************************
** Function name:		wiflyInit
**
** Descriptions:		Initializes Wifly with reset, configures connections,
** 						save changes and reboot it.
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
uint32_t wiflyInit();

/*****************************************************************************
** Function name:		wiflyJoinWifi
**
** Descriptions:		Perform connection with specified SSID
**
** parameters:			cSsid, ulAuthMode, cKeyPhrase
** Returned value:		TRUE if Wifly is online.
**
*****************************************************************************/
uint32_t wiflyJoinWifi(char* ssid, uint32_t authMode, char* keyPhrase);

/*****************************************************************************
** Function name:		wiflyIsJoined
**
** Descriptions:		Check wifi connection.
**
** parameters:			None
** Returned value:		TRUE if Wifly is online
**
*****************************************************************************/
uint32_t wiflyIsJoined();

/*****************************************************************************
** Function name:		wiflyGetIP
**
** Descriptions:		Returns IP address of connection.
**
** parameters:			String to store IP Address.
** Returned value:		None
**
*****************************************************************************/
void wiflyGetIP(char* string);

/*****************************************************************************
** Function name:		wiflyGetChannel
**
** Descriptions:		Returns the channel of wifi connection.
**
** parameters:			String to store channel.
** Returned value:		Channel value.
**
*****************************************************************************/
uint32_t wiflyGetChannel(char* string);

/*****************************************************************************
** Function name:		wiflyGetRSSI
**
** Descriptions:		Returns RSSI info.
**
** parameters:			Signal strength value in string.
** Returned value:		Signal strength value.
**
*****************************************************************************/
int32_t wiflyGetRSSI(char* string);

/*****************************************************************************
** Function name:		wiflyGetUpTime
**
** Descriptions:		Returns power up seconds of Wifly.
**
** parameters:			Power up seconds string.
** Returned value:		Power up seconds.
**
*****************************************************************************/
int32_t wiflyGetUpTime(char* string);

/*****************************************************************************
** Function name:		wiflyPing
**
** Descriptions:		Performs a ping to url.
**
** parameters:			String to store PING result.
** Returned value:		TRUE if ping try is successful.
**
*****************************************************************************/
uint32_t wiflyPing(char* string, char* url);

/*****************************************************************************
** Function name:		wiflyGetTXPower
**
** Descriptions:		Get TX Power info.
**
** parameters:			String to store TX power dBm.
** Returned value:		TX Power value dBm.
**
*****************************************************************************/
uint32_t wiflyGetTXPower(char* string);

/*****************************************************************************
** Function name:		wiflySetTXPower
**
** Descriptions:		Sets TX Power.
**
** parameters:			TX Power to set (0 - 12 dBm).
** Returned value:		TRUE if TX Power set is successful.
**
*****************************************************************************/
uint32_t wiflySetTXPower(int32_t TXPower);


/*****************************************************************************
** Function name:		wiflyDoTCPConnection
**
** Descriptions:		Do TCP connection to stored TCP config
**
** parameters:			Enable auto connection, silentConn remove wifly debug messages
** Returned value:		None
**
*****************************************************************************/
void wiflyDoTCPConnection(uint32_t autoConn, uint32_t silentConn);

/*****************************************************************************
** Function name:		wiflyTCPConf
**
** Descriptions:		Configure TCP connection
**
** parameters:			server_ip & port to connect to
** Returned value:		TRUE if configuration was successful
**
*****************************************************************************/
uint32_t wiflyTCPConf(char* server_ip, char* port);

/*****************************************************************************
** Function name:		wiflyGetHTTP
**
** Descriptions:		Performs HTTP GET with data to webServer.
**
** parameters:			webServer, data
** Returned value:		TRUE if HTTP GET command results successful.
**
*****************************************************************************/
uint32_t wiflyGetHTTP(char* webServer, char* http_get, char* value);

/*****************************************************************************
** Function name:		wiflyGetHTTPValue
**
** Descriptions:		Performs HTTP GET with data to webServer and extracts
** 						value from server response.
**
** parameters:			webServer, http_get, delim1, delim2, offset,  data
** Returned value:		TRUE if HTTP GET command results successful.
**
*****************************************************************************/
uint32_t wiflyGetHTTPValue(char* webServer, char* http_get, char* delim1, char* delim2, uint32_t offset, char* data);

#endif /* WIFLY_H_ */
