/*
 * morse.c
 *  Morse encode - decode functions
 *
 *  Created on: 24/11/2013
 *  Author:  Samuel Melero Cuesta
 *  Version: 1.0
 *
 */

#include "FreeRTOS.h"
#include "queue.h"
#include "LPC17xx.h"
#include "timer.h"
#include "delay.h"
#include "type.h"
#include "pwm.h"
#include "morse.h"
#include "tools.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef struct {
	uint8_t asciiCode; /* ASCII code of character */
	uint8_t morseCode; /* Binary Morse Code */
} xtendedMorseType;

static char morseStandard[] = "##TEMNAIOGKDWRUS##QZYCXBJP#L#FVH";

static xtendedMorseType morseXtended[] = {{48,32},	/* 0 */
										  {49,48}, 	/* 1 */
										  {50,56}, 	/* 2 */
										  {51,60}, 	/* 3 */
										  {52,62}, 	/* 4 */
										  {53,63}, 	/* 5 */
										  {54,47}, 	/* 6 */
										  {55,39}, 	/* 7 */
										  {56,35}, 	/* 8 */
										  {57,33}, 	/* 9 */
										  {209,36}, /* Ñ */
										  {199,43}, /* Ç */
										  {47,45}, 	/* / */
										  {61,46}, 	/* = */
										  {58,71}, 	/* : */
										  {44,76}, 	/* , */
										  {33,84}, 	/* ! */
										  {45,94}, 	/* - */
										  {39,97}, 	/* ' */
										  {64,101}, /* @ */
										  {46,106}, /* . */
										  {34,109}, /* " */
										  {95,114}, /* _ */
										  {63,115}};/* ? */

static size_t standardArrayLength = sizeof(morseStandard) / sizeof(morseStandard[0]);
static size_t xtendedArrayLength = sizeof(morseXtended) / sizeof(xtendedMorseType);
static uint8_t binaryMorse = 1;
static uint32_t pitido, pausa = 0;
static uint32_t dot_time, min_dot_time, max_dot_time;
static uint32_t dash_time, min_dash_time, max_dash_time;
static uint32_t word_time;
static float morse_volume;
static uint32_t morse_freq;
static uint32_t morse_wpm;
static uint32_t pwm_initialized = FALSE;
static xQueueHandle detectorQueue;
portTickType wordPause = 0;

/* Counter 0 value external variable */
extern volatile uint32_t timer0_m0_counter;

/* Local functions */
float calcPWMVolume(float volume);

void EINT3_IRQHandler (void)
{
	portBASE_TYPE xHigherPriorityTaskWoken = pdFALSE;
	static uint8_t morseType;

    if ((LPC_GPIOINT->IO2IntStatR & (1 << 12)) == (1 << 12)) {

    	/* Received rising of signal. Measure timings */
        disable_timer(0);
        pausa = timer0_m0_counter - pitido;
        reset_timer(0);
        timer0_m0_counter = 0;

        LPC_GPIOINT->IO2IntClr |= (1 << 12); // clear the status
        enable_timer(0);

    }
    if ((LPC_GPIOINT->IO2IntStatF & (1 << 12)) == (1 << 12)) {
    	/* Signal was end. Calculate if was dot or dash */
    	pitido = timer0_m0_counter;

		morseType = getMorseSymbol(pausa, pitido);

		/* Check if symbol refers to Morse Code */
		if (morseType != MORSE_NULL) {
			/* Send morse type to decoder task */
			xQueueSendToBackFromISR(detectorQueue, &morseType, &xHigherPriorityTaskWoken);
		}

    	LPC_GPIOINT->IO2IntClr |= (1 << 12); // clear the status
    }
    //Return where we came from
    return;
}

void initMorseReceiver(xQueueHandle receiverQueue)
{
	detectorQueue = receiverQueue;

	// Input PIN Selection & enable interrupt for PIN
    LPC_PINCON->PINSEL4	&= ( ~( 3 << 24 ) );
	LPC_PINCON->PINMODE4 |= ( 3 << 24 );
	LPC_GPIO2->FIODIR &=  ( ~( 1 << 12) );
	LPC_GPIOINT->IO2IntEnF |= (1 << 12);
	LPC_GPIOINT->IO2IntEnR |= (1 << 12);

	init_timer(0, 1);
	enable_timer(0);
	NVIC_EnableIRQ(EINT3_IRQn);
}

void initMorseTransmitter()
{

	morse_freq = MORSE_FREQ_DEF;
	morse_wpm = MORSE_WPM_DEF;
	morse_volume = MORSE_VOLUME_DEF;
	setMorseOptions(NULL);
}

void setMorseOptions(char* options)
{

	if (options != NULL)
		parseOptionData(options, &morse_freq, &morse_wpm, &morse_volume);

	calcWpmTimeBase(morse_wpm);
	if (pwm_initialized != FALSE)
		PWM_Set(morse_freq, calcPWMVolume(morse_volume));
	else
		pwm_initialized = PWM_Init(morse_freq, calcPWMVolume(morse_volume));
}

void getMorseOptions(char* string)
{

	sprintf(string, "!#%u#%u#%u", morse_freq, morse_wpm, (uint32_t)morse_volume);
}

void calcWpmTimeBase(uint32_t wpm)
{
	float time_base = MORSE_WPM_CONST / wpm;
	float range = MORSE_TIME_RANGE * time_base;

	/* Morse code timmings calculation */
	/* Morse time-base = dot time */
	dot_time = (uint32_t)time_base;
	min_dot_time = (uint32_t)(time_base  - range);
	max_dot_time = (uint32_t)(time_base  + range);

	/* Morse dash time */
	dash_time = (uint32_t)(time_base * 3);
	min_dash_time = (uint32_t)(dash_time  - range);
	max_dash_time = (uint32_t)(dash_time  + range);

	/* Morse pause between words */
	word_time = (uint32_t)(time_base * 4);
	wordPause = (portTickType)(word_time + dash_time*2) / portTICK_RATE_MS;
}

float calcPWMVolume(float volume)
{

	return volume *= (0.01 * PWM_VOLUME_CONST);
}

portTickType getMorseWordsPauseTime()
{

	return wordPause;
}

void getBin(uint8_t num, char* str)
{
	*(str+8) = '\0';
	int mask = 0x80;
	uint32_t first_one = FALSE;

	/* Numeric to string binary */
	do {
		if ((mask & num) > 0)
		{
			if (first_one != FALSE)
				*str++ = '1';
			first_one = TRUE;
		}
		else
			if (first_one != FALSE)
				*str++ = '0';

	} while (mask >>= 1);
	*str++ = '\0';
}

uint32_t charPending()
{
	/* Check if Morse buffer contains a character */
	if (binaryMorse > 1)
		return TRUE;
	else
		return FALSE;
}

void shiftBin()
{
	binaryMorse = binaryMorse << 1;
}

void addMorseDot()
{
	/* Add 1 to Morse buffer */
	shiftBin();
	binaryMorse++;
}

void addMorseDash()
{
	/* Add 0 to Morse buffer */
	shiftBin();
}

uint8_t getMorseSymbol(uint32_t pauseTime, uint32_t signalTime)
{
	uint8_t morseSymbol;

	/* Compute Morse symbols from timmings */
	if (pauseTime > dot_time) {
		if ( (pauseTime >= word_time) && (pauseTime < MORSE_PAUSE_MAX) ) {
			if ((signalTime < max_dot_time) && (signalTime > min_dot_time)) // new word dot
				morseSymbol = MORSE_DOT_WORD;
			else
				morseSymbol = MORSE_DASH_WORD;
		}
		else {
			if ((signalTime < max_dot_time) && (signalTime > min_dot_time)) // new char dot
				morseSymbol = MORSE_DOT_CHAR;
			else
				if (signalTime > min_dot_time)
					morseSymbol = MORSE_DASH_CHAR;
				else
					morseSymbol = MORSE_NULL;
		}
	}
	else {
		if ((signalTime < max_dot_time) && (signalTime > min_dot_time)) // dot
			morseSymbol = MORSE_DOT;
		else
			if (signalTime > min_dot_time)
				morseSymbol = MORSE_DASH;
			else
				morseSymbol = MORSE_NULL;
	}

	return morseSymbol;
}

uint8_t decodeXtended(uint8_t morseChar)
{
	uint8_t result;
	uint8_t i = 0;

	/* Decodes no standard Morse code */
	while (i < xtendedArrayLength)
	{
		if (morseChar == morseXtended[i].morseCode)
			break;
		i++;
	}

	if (i < xtendedArrayLength)
		result = morseXtended[i].asciiCode;
	else
		result = '#';

	return result;
}

char decodeChar()
{
	uint8_t result;

	if (binaryMorse < 32)
		result = morseStandard[binaryMorse];
	else {
		result = decodeXtended(binaryMorse);
	}

	/* Reset and prepare binary buffer */
	binaryMorse = 1;
	return result;
}

uint32_t encodeChar(char charToEncode, char* string)
{
	uint8_t bin_char;
	uint8_t i = 0;

	bin_char = (uint8_t)charToEncode;
	if (charToEncode == 135)
		bin_char = 128;
	if (charToEncode == 145)
		bin_char = 165;
	if (bin_char > 64 && bin_char < 91)	/* Standard morse code array */
	{
		while (i < standardArrayLength)
		{
			if (morseStandard[i] == charToEncode)
				break;
			i++;
		}
		bin_char = i;
	}
	else
	{
		while (i < xtendedArrayLength) {
			if (morseXtended[i].asciiCode == bin_char)
				break;
			i++;
		}
		bin_char = morseXtended[i].morseCode;
	}

	getBin(bin_char, string);

	return strlen(string);
}

void doMorseSound(char charToEncode)
{
	char morseSymbols[9];
	uint32_t i, symbols_num;


	if (charToEncode == ' ')
		passiveDelay(word_time); 			/* pause between words */
	else {
		symbols_num = encodeChar(charToEncode, morseSymbols);
		for (i=0; i < symbols_num; i++)
		{
			PWM_Start();
			if (morseSymbols[i] == '1')
				passiveDelay(dot_time); 	/* dot time */
			else
				passiveDelay(dash_time); 	/* dash time */
			PWM_Stop();
			passiveDelay(dot_time);			/* pause between symbols */
		}
		passiveDelay(dash_time); 			/* dash time pause between chars */
	}
}
