/*
 * wifly.h
 *  Wifly driver
 *
 *  Created on: 24/10/2013
 *  Author:  Samuel Melero Cuesta
 *  Version: 2.0
 *
 */

#include "LPC17xx.h"
#include "type.h"
#include "config.h"
#include "uart.h"
#include "uartstr.h"
#include "delay.h"
#include "tools.h"
#include "wifly.h"
#include "sync.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char rx_buffer[BUFSIZE];
static uint32_t wf_comm_mode = FALSE;
static uint32_t wf_config_HTTP = FALSE;

uint32_t wiflyGetBuffer(char* string, uint32_t timeout) {

	return UART0Scanf(string, timeout);
}

/*****************************************************************************
** Function name:		wiflyCheckResp
**
** Descriptions:		Look for expected value in rx buffer.
**
** parameters:			expected value, listen time.
** Returned value:		TRUE, FALSE
**
*****************************************************************************/
uint32_t wiflyCheckResp(char* expected, uint32_t timeout) {

	UART0Scanf(rx_buffer, timeout);
	if (strstr(rx_buffer, expected) != NULL)
		return ( TRUE );
	else
		return ( FALSE) ;
}

/*****************************************************************************
** Function name:		wiflySendCommValue
**
** Descriptions:		Send command with value and checks expected
** 						response.
**
** parameters:			command, value, expected, timeout
** Returned value:		TRUE if the response is the expected value.
**
*****************************************************************************/
uint32_t wiflySendCommValueCheck(char* command, char* value, char* expected, uint32_t timeout) {

	printfUart(WF_UART_PORT, "%s %s\r", command, value);
	wiflyCheckResp(expected, timeout);
	if (wiflyCheckResp(expected, timeout) != FALSE)
		return ( TRUE );
	else
		return ( FALSE );
}

/*****************************************************************************
** Function name:		wiflySendCommValue
**
** Descriptions:		Send command with value with no check.
**
** parameters:			command, value
** Returned value:		None
**
*****************************************************************************/
void wiflySendCommValue(char* command, char* value) {

	printfUart(WF_UART_PORT, "%s %s\r", command, value);
	activeDelay(WF_STDCOMM_TIME);
}

/*****************************************************************************
** Function name:		wiflySendCommCheck
**
** Descriptions:		Send command and checks expected response.
**
** parameters:			command, expected, timeout
** Returned value:		TRUE if the response is the expected value.
**
*****************************************************************************/
uint32_t wiflySendCommCheck(char* command, char* expected, uint32_t timeout) {

	printfUart(WF_UART_PORT, "%s\r", command);
	if (wiflyCheckResp(expected, timeout) != FALSE)
		return ( TRUE );
	else
		return ( FALSE );
}

/*****************************************************************************
** Function name:		wiflySendComm
**
** Descriptions:		Send command.
**
** parameters:			cWFCommand
** Returned value:		None
**
*****************************************************************************/
void wiflySendComm(char* command) {

	printfUart(WF_UART_PORT, "%s\r", command);
	activeDelay(WF_STDCOMM_TIME);
}

/*****************************************************************************
** Function name:		wiflySendCommNum
**
** Descriptions:		Send command with numeric value.
**
** parameters:			cWFCommand, cWFValue
** Returned value:		TRUE if all gone OK.
**
*****************************************************************************/
void wiflySendCommNum(char* command, int32_t value) {

	printfUart(WF_UART_PORT, "%s %d\r", command, value);
}

/*****************************************************************************
** Function name:		wiflySendCommNumCheck
**
** Descriptions:		Send command with numeric value and checks expected
** 						response.
**
** parameters:			command, value, expected, timeout
** Returned value:		TRUE if all gone OK.
**
*****************************************************************************/
uint32_t wiflySendCommNumCheck(char* command, int32_t value, char* expected, uint32_t timeout) {

	printfUart(WF_UART_PORT, "%s %d\r", command, value);
	if (wiflyCheckResp(expected, timeout) != FALSE)
		return ( TRUE );
	else
		return ( FALSE );
}

/*****************************************************************************
** Function name:		wiflyReset
**
** Descriptions:		Resets the Wifly.
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void wiflyReset() {
	/* During Wifly initialization, it must be reset */

	/* Initialize pin */
	LPC_PINCON->PINSEL1 &= ( ~(3 << RST_WIFLY_BIT) );
	LPC_GPIO0->FIODIR |= ( 1 << RST_WIFLY_PIN );

	/* Reset the Wifly */
	LPC_GPIO0->FIOCLR = 1 << RST_WIFLY_PIN;
	activeDelay(1);
	LPC_GPIO0->FIOSET = 1 << RST_WIFLY_PIN;
}

/*****************************************************************************
** Function name:		wiflyConfGPIO
**
** Descriptions:		Configures wifly to use GPIO 4 and 6 (WLAN and TCP status)
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void wiflyConfGPIO()
{
	/* Initialize wifly status PINs */
	LPC_PINCON->PINSEL1 &= ( ~(3 << WLAN_WIFLY_BIT) );
	LPC_PINCON->PINMODE1 |= (3 << WLAN_WIFLY_BIT);
	LPC_GPIO0->FIODIR &= ( ~(1 << WLAN_WIFLY_PIN) );

	LPC_PINCON->PINSEL1 &= ( ~(3 << TCP_WIFLY_BIT) );
	LPC_PINCON->PINMODE1 |= (3 << TCP_WIFLY_BIT);
	LPC_GPIO0->FIODIR &= ( ~(1 << TCP_WIFLY_PIN) );
}

uint32_t wiflyGetTCPStatus()
{
	unsigned long pinStatus;

	/* Obtain the current state. */
	pinStatus = LPC_GPIO0->FIOPIN;

	/* Get the TCP connection status */
	if ( !( pinStatus & (1 << TCP_WIFLY_PIN) ) )
		return FALSE;
	else
		return TRUE;
}

uint32_t wiflyGetWLANStatus()
{
	unsigned long pinStatus;

	/* Obtain the current state. */
	pinStatus = LPC_GPIO0->FIOPIN;

	/* Get the WLAN connection status */
	if ( !( pinStatus & (1 << WLAN_WIFLY_PIN) ) )
		return FALSE;
	else
		return TRUE;
}

/*****************************************************************************
** Function name:		wiflyEnterCMD
**
** Descriptions:		Enter CMD mode.
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void wiflyEnterCMD() {
	uint8_t uiRetry = 10;
	uint32_t response = FALSE;

	do {
		uiRetry--;
		printfUart(WF_UART_PORT, "%s", WF_CMD_ENTER);
		response = wiflyCheckResp("CMD", 300);
	} while ( (response != TRUE) && (uiRetry > 0) );

	printfUart(WF_UART_PORT, "%c", '\r');
	passiveDelay(250);

	wf_comm_mode = TRUE;
}

/*****************************************************************************
** Function name:		wiflyExitCMD
**
** Descriptions:		Exit from CMD mode.
**
** parameters:			None
** Returned value:		TRUE if Wifly Exits from CMD mode.
**
*****************************************************************************/
void wiflyExitCMD() {

	if (wf_comm_mode != FALSE) {
		printfUart(WF_UART_PORT, "%s\r", WF_CMD_EXIT);
		if (wiflyCheckResp("EXIT", WF_STDCOMM_TIME) != FALSE)
			wf_comm_mode = FALSE;
	}
}

uint32_t wiflyInit() {
	uint32_t response = FALSE;

	if (printfInit(WF_UART_PORT, WF_UART_BAUDRATE) != FALSE) {
		printfFlush(WF_UART_PORT);
		wiflyReset();
		passiveDelay(WF_RESET_TIME);
		wiflyConfGPIO();
		passiveDelay(250);

		/* Configure wifly params, save and reboot */
		wiflyEnterCMD();
		//passiveDelay(3000);
		wiflySendCommValue("set sys iofunc", "0x50");
		passiveDelay(WF_STDCOMM_TIME);
		wiflySendComm("save");
		passiveDelay(WF_STDCOMM_TIME * 2);
		wiflySendComm("reboot");
		passiveDelay(WF_RESET_TIME);

		wf_config_HTTP = FALSE;
		wf_comm_mode = FALSE;
		response = TRUE;
	}

	return response;
}


uint32_t wiflyJoinWifi(char* ssid, uint32_t authMode, char* keyPhrase) {
	uint32_t result = FALSE;
	uint32_t wait = 10000;
	char c_SSID[strlen(ssid)];

	wiflyEnterCMD();
	if (wf_comm_mode != FALSE ) {
		strcpy(c_SSID, ssid);
		cpChangeBlanks(c_SSID);

			if (authMode != WF_WEP_MODE)
			switch(authMode) {
				case (WF_WPA_MODE):
					result = wiflySendCommValueCheck("set wlan phrase", keyPhrase, "AOK", WF_STDCOMM_TIME);
					break;
				case (WF_WEP_MODE):
					result = wiflySendCommValueCheck("set wlan num", keyPhrase, "AOK", WF_STDCOMM_TIME);
					break;
				default:
					result = TRUE;
			}
			result &= wiflySendCommValueCheck("set wlan ssid", c_SSID, "AOK", WF_STDCOMM_TIME);

			if (result != FALSE) {
				wiflySendComm("join");
				passiveDelay(3000);
				do {
					result = wiflyGetWLANStatus();
					wait--;
					passiveDelay(WF_STDCOMM_TIME);
				} while ( (result != TRUE) && (wait > 0) );
			}
	}

	return result;
}


uint32_t wiflyIsJoined() {
	uint32_t result = FALSE;
	char string[WF_MAX_CMD_SIZE];

	wiflyLock();
	wiflyEnterCMD();
	printfUart(WF_UART_PORT, "%s\r", "show net");
	UART0Scanf(rx_buffer, WF_STDCOMM_TIME);
	vDelimChars(rx_buffer, string, "Assoc=", "\r\n", 6);
	if (strstr(string, "OK") != NULL) {
		vDelimChars(rx_buffer, string, "Auth=", "\r\n", 5);
		if (strstr(string, "OK") != NULL)
			result = TRUE;
	}
	wiflyUnlock();
	return result;
}


void wiflyGetIP(char* string) {

	wiflyLock();
	wiflyEnterCMD();
	printfUart(WF_UART_PORT, "%s\r", "get ip");
	UART0Scanf(rx_buffer, WF_STDCOMM_TIME);
	vDelimChars(rx_buffer, string, "IP=", ":", 3);
	wiflyUnlock();
}


uint32_t wiflyGetChannel(char* string) {

	wiflyLock();
	wiflyEnterCMD();
	printfUart(WF_UART_PORT, "%s\r", "show net");
	UART0Scanf(rx_buffer, WF_STDCOMM_TIME);
	vDelimChars(rx_buffer, string, "Chan=", "\r\n", 5);
	wiflyExitCMD();
	wiflyUnlock();

	return atoi(string);
}


int32_t wiflyGetRSSI(char* string) {

	wiflyLock();
	wiflyEnterCMD();
	printfUart(WF_UART_PORT, "%s\r", "show rssi");
	UART0Scanf(rx_buffer, WF_STDCOMM_TIME);
	vDelimChars(rx_buffer, string, "RSSI=", "\r\n", 0);
	vDelimChars(rx_buffer, string, "(", ")", 1);
	wiflyExitCMD();
	wiflyUnlock();

	return atoi(string);
}


int32_t wiflyGetUpTime(char* string) {

	wiflyLock();
	wiflyEnterCMD();
	printfUart(WF_UART_PORT, "%s\r", "show time");
	UART0Scanf(rx_buffer, WF_STDCOMM_TIME);
	vDelimChars(rx_buffer, string, "UpTime=", "\r\n", 0);
	vDelimChars(rx_buffer, string, "=", "=", 1);
	wiflyExitCMD();
	wiflyUnlock();

	return atoi(string);
}


uint32_t wiflyPing(char* string, char* url) {
	uint32_t response = FALSE;

	wiflyLock();
	wiflyEnterCMD();
	printfFlush(WF_UART_PORT);
	response = wiflySendCommValueCheck("ping", url, "ms", 3000);
	strcpy(string, rx_buffer); /* Store result anyway, for debug purposes */
	wiflyExitCMD();
	wiflyUnlock();

	return response;
}


uint32_t wiflyGetTXPower(char* string)
{
	uint32_t txPower = 0;

	wiflyLock();
	wiflyEnterCMD();
	printfUart(WF_UART_PORT, "%s\r", "get wlan");
	UART0Scanf(rx_buffer, WF_STDCOMM_TIME);
	vDelimChars(rx_buffer, string, "TxPower=", "\r\n", 8);
	txPower = atoi(string);

	/* El valor 0 es igual al máximo valor, 12 dBm */
	if (txPower == 0) {
		txPower = 12;
		string = "12";
	}
	wiflyExitCMD();
	wiflyUnlock();

	return txPower;
}


uint32_t wiflySetTXPower(int32_t TXPower)
{

	uint32_t response = FALSE;

	wiflyLock();
	wiflyEnterCMD();
	if (TXPower <= 12)
		response = wiflySendCommNumCheck("set wlan tx", TXPower, "AOK", WF_STDCOMM_TIME);
	wiflyExitCMD();
	wiflyUnlock();

	return response;
}

void wiflyDoTCPConnection(uint32_t autoConn, uint32_t silentConn)
{
	wiflyLock();
	wiflyEnterCMD();
	if (silentConn != FALSE)
	{
		wiflySendCommValue("set comm open", "0");
		wiflySendCommValue("set comm close", "0");
	}
	if (autoConn != FALSE)
		wiflySendCommValue("set sys autoconn", "3");
	wiflySendComm("open");
	wiflyUnlock();
}

uint32_t wiflyTCPConf(char* server_ip, char* port)
{
	uint32_t response = FALSE;

	wiflyLock();
	wiflyEnterCMD();
	if (wiflySendCommValueCheck("set ip proto", "2", "AOK", WF_STDCOMM_TIME) != FALSE)
		if (wiflySendCommValueCheck("set ip host", server_ip, "AOK", WF_STDCOMM_TIME) != FALSE)
			if (wiflySendCommValueCheck("set ip remote", port, "AOK", WF_STDCOMM_TIME) != FALSE)
				if (wiflySendCommValueCheck("set comm remote", "0", "AOK", WF_STDCOMM_TIME) != FALSE)
					if (wiflySendCommValueCheck("set comm size", "1420", "AOK", WF_STDCOMM_TIME) != FALSE)
						if (wiflySendCommValueCheck("set comm time", "100", "AOK", WF_STDCOMM_TIME) != FALSE)
							response = TRUE;
	wiflyUnlock();
	return response;
}

uint32_t wiflyGetHTTP(char* webServer, char* http_get, char* value) {
	uint32_t response = FALSE;
	char cGET[strlen(http_get)];

	wiflyEnterCMD();

	/* Change blanks by blank command mode char */
	strcpy(cGET, http_get);
	cpChangeBlanks(cGET);
	/********************************************/

	/* Check if HTTP mode config was entered, it's not necessary to config it again */
	if (wf_config_HTTP != TRUE) {
		if (wiflySendCommValueCheck("set ip proto", "18", "AOK", WF_STDCOMM_TIME) != FALSE)
			if (wiflySendCommValueCheck("set dns name", webServer, "AOK", WF_STDCOMM_TIME)!= FALSE)
				if (wiflySendCommValueCheck("set ip address", "0", "AOK", WF_STDCOMM_TIME)!= FALSE)
					if (wiflySendCommValueCheck("set ip remote", "80", "AOK", WF_STDCOMM_TIME)!= FALSE)
						response = TRUE;
	}
	else
		response = TRUE;

	if (response != FALSE) {
		response = FALSE;
		if (wiflySendCommValueCheck("set com remote", cGET, "AOK", WF_STDCOMM_TIME)!= FALSE)
			if (wiflySendCommValueCheck("open", "", "OPEN", WF_OPENTCP_TIME) != FALSE)
				if (wiflySendCommCheck(value, "200 OK", 800) != FALSE) {
					response = ( TRUE );
					if (strstr(rx_buffer, "*CLOS") != NULL)
						wf_comm_mode = FALSE;
				}
	}

	wf_config_HTTP = response; /* If all is OK then config HTTP is OK */
	return response;
}


uint32_t wiflyGetHTTPValue(char* webServer, char* http_get, char* delim1, char* delim2, uint32_t offset, char* data) {

	if (wiflyGetHTTP(webServer, http_get, "") != FALSE) {
		vDelimChars(rx_buffer, data, delim1, delim2, offset);
		return ( TRUE );
	}
	else
		return ( FALSE );
}
