/*
===============================================================================
 Name        : main.c
 Author      : Samuel Melero Cuesta
 Version     : 1.0
 Copyright   : Copyright (C)
 Description : main definition
===============================================================================
*/

#include "LPC17xx.h"
#include "FreeRTOS.h"
#include "queue.h"
#include "task.h"
#include "type.h"
#include "morse.h"
#include "pwm.h"
#include "delay.h"
#include "uart.h"
#include "uartstr.h"
#include "tools.h"
#include "wifly.h"
#include "sync.h"
#include "config.h"

/* Wifi engine values */
uint32_t wlan_protected = TRUE;
uint32_t ssid_pass_type = WF_NO_PROTECTED;
char ssid_pass[256];
char ssid[60];
char host_ip[20];
static char* host_port = "6001";

/* Morse engine values */
char morse_settings[20];

/* Tasks data */
char receiveBuff[BUFSIZE];
static void txtReceiveTask( void *pvParameters );
static void morseSoundTask( void *pvParameters );
static void morseReceiverTask( void *pvParameters );
static void txtSenderTask( void *pvParameters );
static xQueueHandle morseSoundQueue;
static xQueueHandle morseReceiveQueue;
static xQueueHandle charSendQueue;

/* Function prototypes */
uint32_t doConnection();
void waitForConnection();
void retryConnection();
uint32_t checkConnection();
void decodeAndSend(uint32_t isChar);
void printMenu();
void checkMenuResponse(const char* question, char* string);
void sendStatus(char* status);

/*-----------------------------------------------------------*/

/*
 * MAIN APPLICATION
 */
int main(void) {

	uint32_t response = FALSE;

	initHandlers();
	printfInitDebug();
	printMenu();
	response = doConnection();
	if (response != TRUE)
		retryConnection();
	response = checkConnection();
	passiveDelay(1000);

	morseSoundQueue = xQueueCreate( 16, sizeof(char) );
	morseReceiveQueue = xQueueCreate( 10, sizeof(uint8_t) );
	charSendQueue = xQueueCreate( 10, sizeof(char) );

	if( (morseSoundQueue != NULL) && (response != FALSE) )
	{
		initMorseReceiver(morseReceiveQueue);
		initMorseTransmitter();

		/* Send settings to user application */
		getMorseOptions(morse_settings);
		printfUart(WF_UART_PORT, "%s\r", morse_settings);

		xTaskCreate( txtReceiveTask, (signed char *)"charInput", 1024, NULL, 1, NULL );
		xTaskCreate( morseSoundTask, (signed char *)"soundMorse", 1024, NULL, 1, NULL );
		xTaskCreate( morseReceiverTask, (signed char *)"morseDecoder", 1024, NULL, 1, NULL );
		xTaskCreate( txtSenderTask, (signed char *)"charSender", 1024, NULL, 1, NULL );

		/* Start the scheduler so the created tasks start executing. */
		vTaskStartScheduler();
	}
	else
	{
		/* Program fail to start */
	}

	printfDebug("%s", "ERROR");
	for( ;; );

	return 0;
}

/*-----------------------------------------------------------*/

static void morseReceiverTask( void *pvParameters )
{
	portBASE_TYPE xStatus;
	uint8_t morseValue;
	portTickType timeWaitForQueue = portMAX_DELAY;

	/*
	 * This task performs listen from audio input.
	 * It's synchronized with EINT3
	 */

	for( ;; )
	{

		xStatus = xQueueReceive(morseReceiveQueue, &morseValue, timeWaitForQueue);
		disableUART0Int(); /* Disable incoming from wifi. Transmitter is turned off */
		if( xStatus == pdPASS )
		{
			/* Task will not wait indefinitely if there are pending chars in queue */
			timeWaitForQueue = getMorseWordsPauseTime();

			/* Perform action from timings received */
			switch (morseValue)
			{
				case MORSE_DOT:
					addMorseDot();
					break;

				case MORSE_DASH:
					addMorseDash();
					break;

				case MORSE_DOT_WORD:
					if (charPending() == TRUE)
						decodeAndSend(FALSE);
					addMorseDot();
					break;

				case MORSE_DASH_WORD:
					if (charPending() == TRUE)
						decodeAndSend(FALSE);
					addMorseDash();
					break;

				case MORSE_DOT_CHAR:
					if (charPending() == TRUE)
						decodeAndSend(TRUE);
					addMorseDot();
					break;

				case MORSE_DASH_CHAR:
					if (charPending() == TRUE)
						decodeAndSend(TRUE);
					addMorseDash();
			}
		}
		else
		{
			/* No more Morse bins received out. Time to send pending char */
			if (charPending() == TRUE) {
				decodeAndSend(FALSE);

				/* Entire word was sent so it's time to wait indefinitely */
				timeWaitForQueue = portMAX_DELAY;
			}
		}
		enableUART0Int(); /* Enable incoming from wifi */
	}
}

/*-----------------------------------------------------------*/

static void txtSenderTask( void *pvParameters )
{
	portBASE_TYPE xStatus;
	char charToSend;

	/*
	 * This task sends characters received from Morse code audio
	 */

	for (;;) {
		xStatus = xQueueReceive(charSendQueue, &charToSend, portMAX_DELAY);
		if( xStatus == pdPASS )
			printfUart(WF_UART_PORT, "%c\r", charToSend);
	}
}

/*-----------------------------------------------------------*/

static void txtReceiveTask( void *pvParameters )
{
	uint32_t i, buff_length;

	/*
	 * This task performs listen from wifi connection
	 * It receives data that should be transformed in Morse code
	 * It's synchronized with INT_UART0
	 */

	for( ;; )
	{
		lockUART0();
		if (checkConnection() != TRUE)
			retryConnection();

		receiveBuff[0] = '\0';
		buff_length = wiflyGetBuffer(receiveBuff, 250);
		/* Check for wifly system messages */
		if (strContains(receiveBuff, "*!")) {
			for (i=2; i < buff_length; i++) {
				/* Disable receiver while transmitter is running */
				NVIC_DisableIRQ(EINT3_IRQn);
				xQueueSendToBack(morseSoundQueue, &receiveBuff[i], portMAX_DELAY);
			}
		}
		else
			if (strContains(receiveBuff, "!*")) {
				/* Option data received from application */
				setMorseOptions(receiveBuff);
			}
	}
}
/*-----------------------------------------------------------*/

static void morseSoundTask( void *pvParameters )
{
	char charToEncode;

	/*
	 * This task performs Morse sounds from text
	 * It receives chars that should be transformed in Morse code
	 * Morse code sounds are launched through PWM signal
	 */

	for( ;; )
	{
		xQueueReceive(morseSoundQueue, &charToEncode, portMAX_DELAY);
		doMorseSound(charToEncode);
		if (uxQueueMessagesWaiting(morseSoundQueue) == 0) {
			sendStatus(LPC_STATUS_OK);
			/* Enable receiver, transmission is done */
			NVIC_EnableIRQ(EINT3_IRQn);
		}
	}
}
/*-----------------------------------------------------------*/

void decodeAndSend(uint32_t isChar)
{
	/*
	 * This functions performe decode of incoming char from Morse
	 * and sends it to sender task.
	 */
	char decodedChar = decodeChar();
	xQueueSendToBack(charSendQueue, &decodedChar, portMAX_DELAY);

	/* If not 'isChar' it is word. Send space to finish word */
	if (isChar != TRUE) {
		decodedChar = ' ';
		xQueueSendToBack(charSendQueue, &decodedChar, portMAX_DELAY);
	}
}

/*-----------------------------------------------------------*/

uint32_t doConnection()
{
	uint32_t response;

	/*
	 * This functions performs hard connection and wifly reset
	 */

	passiveDelay(4000);
	printfDebug("\r\n%s ", "Initializing Wifly, please wait...");
	wiflyInit();
	printfDebug("%s\r\n", "OK");
	printfDebug("%s ", "Trying to join SSID, please wait...");
	response = wiflyJoinWifi(ssid, ssid_pass_type, ssid_pass);

	if (response != FALSE)
		printfDebug("%s\r\n", "OK");
	else
		printfDebug("%s\r\n", "NOK");

	passiveDelay(500);

	printfDebug("%s ", "Trying to connect to server...");
	response &= wiflyTCPConf(host_ip, host_port);
	if (response != FALSE)
		wiflyDoTCPConnection(TRUE, TRUE);

	response = checkConnection();
	if (response != FALSE)
		printfDebug("%s\r\n", "OK");
	else
		printfDebug("%s\r\n", "NOK");

	return response;
}

/*-----------------------------------------------------------*/

void waitForConnection()
{
	uint32_t response;
	uint32_t retry;

	/*
	 * This function waits 2 minutes while checks connection status
	 */

	retry = 60; /* 2 minutos de reintento */
	do {
		passiveDelay(2000);
		response = checkConnection();
		retry--;
	} while (response != TRUE && retry > 0);
}

/*-----------------------------------------------------------*/

void retryConnection()
{
	uint32_t response;

	/*
	 * This function wait for autoconnection and performs a hard
	 * reconnection if wait fails
	 */

	do{
		printfDebug("\r\n%s", "Trying to connect...");
		waitForConnection();
		response = checkConnection();
		if (response != TRUE) {
			printfDebug("%s\r\n", "NOK");
			response = doConnection();
		}
		else
			printfDebug("%s\r\n", "OK");
	} while (response != TRUE);
}

/*-----------------------------------------------------------*/

uint32_t checkConnection()
{
	/*
	 * This functions returns TRUE if wireless connection is OK
	 * FALSE otherwise. It checks the wifly GPIO Status.
	 */
	return wiflyGetWLANStatus() && wiflyGetTCPStatus();
}

/*-----------------------------------------------------------*/

void printMenu()
{
	char resp_buffer[60];

	/*
	 * Application menu. It is launched through debug UART port 3
	 * Performs wifi connection configuration.
	 */

	printfDebug("%s\r\n", "*************************************");
	printfDebug("%s\r\n", "********* WEMAT CONFIG MENU *********");
	printfDebug("%s\r\n\r\n", "*************************************");

	printfDebug("%s\r\n", "WIFI CONNECTION SETTINGS:");
	printfDebug("%s\r\n\r\n", "Note: Data must be entered with CRLF termination.");

	resp_buffer[0] = '\0';
	while (strContains(resp_buffer, "y") != TRUE) {
		checkMenuResponse("\r\nEnter SSID:", resp_buffer);
		strCopy(resp_buffer, ssid);

		checkMenuResponse("\r\nIs WLAN password protected?(y/n):", resp_buffer);
		if (strContains(resp_buffer, "y") != FALSE) {
			wlan_protected = TRUE;
			checkMenuResponse("\r\nIs WPA protected?(y/n):", resp_buffer);
			if (strContains(resp_buffer, "y") != FALSE) {
				checkMenuResponse("\r\nEnter WPA phrase:", resp_buffer);
				ssid_pass_type = WF_WPA_MODE;
			}
			else {
				checkMenuResponse("\r\nEnter WEP key:", resp_buffer);
				ssid_pass_type = WF_WEP_MODE;
			}
			strCopy(resp_buffer, ssid_pass);
		}
		else {
			wlan_protected = FALSE;
			ssid_pass_type = WF_NO_PROTECTED;
		}

		checkMenuResponse("\r\nEnter host application IP:", resp_buffer);
		strCopy(resp_buffer, host_ip);

		printfDebug("\r\n\r\n%s %s\r\n", "Entered SSID:", ssid);
		if (wlan_protected != FALSE) {
			printfDebug("%s %s\r\n", "Entered WLAN Password:", ssid_pass);
		}
		printfDebug("%s %s\r\n", "Entered host IP:", host_ip);

		checkMenuResponse("Is entered data OK?(y/n):", resp_buffer);
	}
}

/*-----------------------------------------------------------*/

void checkMenuResponse(const char* question, char* string)
{
	/*
	 * Application menu. This function launchs a question to screen
	 * and retrieves response from user (saved in string param).
	 */

    printfDebug("%s ", question);
    string[0] = '\0';
    printfFlush(3);
    while (strContainsCRLF(string) != TRUE){
    	UART3Scanf(string, 1);
    }
    strRemoveCRLF(string);
}

/*-----------------------------------------------------------*/

void sendStatus(char* status)
{
	/*
	 * This function send LPC status to user application
	 */
	printfUart(WF_UART_PORT, "¡#%s\r", status);
}

/*-----------------------------------------------------------*/

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */
	for( ;; );
}
/*-----------------------------------------------------------*/
